/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import React, { ReactElement, useEffect, useState } from "react";
import "./assets/App.css";
import { ApplicationFrame } from "./components/Core/Application Frame/ApplicationFrame";
import { useTranslateFunction } from "./hooks/useTranslate";
import { User } from "./api/types";
import {
  getCurrentUserFromLocalStorage,
  loginAndStartTokenRefresher,
  redirectToSSO,
  userTokenRefresher
} from "./api/auth/auth.service";
import { NavigateFunction, useNavigate } from "react-router-dom";

export interface AppProps {
  mainContent: ReactElement;
}

export function App(props: AppProps) {
  const navigate: NavigateFunction = useNavigate();
  const translate = useTranslateFunction();
  const [authenticated, setAuthenticated] = useState(false);
  const [loading, setLoading] = useState(false);
  const [authError, setAuthError] = useState(false);
  const [currentUser, setCurrentUser] = useState<User | undefined>(undefined);

  const setStatesForUser = (user: User) => {
    setCurrentUser(user);
    setAuthError(false);
    setAuthenticated(true);
  };

  useEffect(() => {
    setLoading(true);
    const user = getCurrentUserFromLocalStorage();
    if (user) {
      userTokenRefresher(user);
      setStatesForUser(user);
    } else {
      const searchParams = new URLSearchParams(document.location.search);
      const authCode = searchParams.get("code");
      if (authCode) {
        loginAndStartTokenRefresher(authCode).then(
          (user) => {
            setStatesForUser(user);
            navigate("/");
          },
          (error: { response: { data: { message: any } }; message: any; toString: () => any }) => {
            console.log(error);
            setAuthError(true);
          }
        );
      } else {
        setAuthError(false);
        setAuthenticated(false);
        redirectToSSO();
      }
    }
    setLoading(false);
  }, []);

  if (loading) {
    return <div>{translate("auth.loading")}</div>;
  }

  if (authError) {
    return <div>{translate("auth.error")}</div>;
  }

  if (authenticated) {
    if (!currentUser) {
      return <div>{translate("auth.error")}</div>;
    } else {
      return (
        <ApplicationFrame
          user={currentUser}
          privacyPolicyLabel={translate("common.privacyPolicy")}
          imprintLabel={translate("common.imprint")}
          pageHeaderAriaLabel={translate("common.pageHeaderAriaLabel")}
          logoAltText={translate("common.logoAltText")}
          userIconAltText={translate("common.userIconAltText")}
          bayernIdIconAltText={translate("common.bayernIdIconAltText")}
          ozgCloudIconAltText={translate("common.ozgCloudIconAltText")}
        >
          {props.mainContent}
        </ApplicationFrame>
      );
    }
  } else {
    return <>{translate("auth.loading")}</>;
  }
}
