/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import axios from "axios";
import { downloadAttachment, saveDownloadedAttachmentToFile, uploadAttachment } from "./attachmentsAPI";

describe("Attachments API tests", () => {
  afterEach(() => {
    jest.clearAllMocks(); // reset call counts
  });
  test("attachment upload valid attachment", async () => {
    const mockUploadAttachment = jest.spyOn(axios, "post").mockReturnValue(Promise.resolve({ data: "new_id" }));
    const result = await uploadAttachment("1", "2", new File(["test"], "test.png", { type: "image/png" }));
    expect(result).toEqual("new_id");
    expect(mockUploadAttachment).toBeCalledTimes(1);
  });
  test("attachment upload invalid size", async () => {
    const mockUploadAttachment = jest.spyOn(axios, "post").mockReturnValue(Promise.resolve({ data: "new_id" }));
    const mockedBigFile = new File(["x"], "test.png", { type: "image/png" });
    Object.defineProperty(mockedBigFile, "size", { value: 1024 * 1024 * 120 });

    await expect(uploadAttachment("1", "2", mockedBigFile)).rejects.toEqual(Error("attachments.error.fileSize"));
    expect(mockUploadAttachment).toBeCalledTimes(0);
  });
  test("attachment upload invalid filetype", async () => {
    const mockUploadAttachment = jest.spyOn(axios, "post").mockReturnValue(Promise.resolve({ data: "new_id" }));
    await expect(uploadAttachment("1", "2", new File(["test"], "test.txt", { type: "text/plain" }))).rejects.toEqual(
      Error("attachments.error.fileType")
    );
    expect(mockUploadAttachment).toBeCalledTimes(0);
  });
  test("attachment upload error", async () => {
    const mockUploadAttachment = jest.spyOn(axios, "post").mockRejectedValue("some network error");
    const mockIsAxiosError = jest.spyOn(axios, "isAxiosError").mockReturnValue(false);
    await expect(uploadAttachment("1", "2", new File(["test"], "test.png", { type: "image/png" }))).rejects.toEqual(
      Error("attachments.error.network")
    );
    expect(mockUploadAttachment).toBeCalledTimes(1);
    expect(mockIsAxiosError).toBeCalledTimes(1);
  });
  test("attachment upload virus error", async () => {
    const mockUploadAttachment = jest.spyOn(axios, "post").mockRejectedValue({ response: { status: 406 } });
    const mockIsAxiosError = jest.spyOn(axios, "isAxiosError").mockReturnValue(true);
    await expect(uploadAttachment("1", "2", new File(["test"], "test.png", { type: "image/png" }))).rejects.toEqual(
      Error("attachments.error.virus")
    );
    expect(mockUploadAttachment).toBeCalledTimes(1);
    expect(mockIsAxiosError).toBeCalledTimes(1);
  });
  test("attachment download success", async () => {
    const mockDownloadAttachment = jest.spyOn(axios, "get").mockReturnValue(Promise.resolve({ data: "asdf" }));
    const result = await downloadAttachment("1", "2", "3");
    console.log("what", result);
    const resultText = await result.text();
    expect(resultText).toEqual("asdf");
    expect(mockDownloadAttachment).toBeCalledTimes(1);
  });
  test("attachment download error", async () => {
    const mockDownloadAttachment = jest.spyOn(axios, "get").mockRejectedValue("some network error");
    await expect(downloadAttachment("1", "2", "3")).rejects.toEqual(Error("attachments.error.network"));
    expect(mockDownloadAttachment).toBeCalledTimes(1);
  });
  test("trigger file save for downloaded file", () => {
    const temporaryLink = {
      click: jest.fn()
    };
    global.URL.createObjectURL = jest.fn();
    global.URL.revokeObjectURL = jest.fn();
    jest.spyOn(document, "createElement").mockImplementation(() => temporaryLink as unknown as HTMLAnchorElement);
    saveDownloadedAttachmentToFile(new Blob(["asdf"]), "test.png");

    expect((temporaryLink as unknown as HTMLAnchorElement).download).toEqual("test.png");
    expect(global.URL.createObjectURL).toBeCalledTimes(1);
    expect(temporaryLink.click).toBeCalledTimes(1);
    expect(global.URL.revokeObjectURL).toBeCalledTimes(1);
  });
});
