/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  GET_RUECKFRAGE_FAILED,
  GET_RUECKFRAGE_HEADERS_FAILED,
  getRueckfrageHeadersOfPostfach,
  getRueckfrageOfPostfach,
  SEND_FAILED,
  sendMessage
} from "./rueckfragenAPI";
import { Antwort, Command, CommandReference, Rueckfrage, RueckfrageHeader } from "../types";
import axios from "axios";

jest.mock("axios");

describe("Rueckfrage API tests", () => {
  afterEach(() => {
    jest.clearAllMocks(); // reset call counts
  });

  describe("get topic headers of postfach tests", () => {
    const rueckfrageHeader: RueckfrageHeader = {
      id: "1",
      title: "test topic",
      nachrichtHeader: {
        id: "1",
        vorgangId: "1",
        sendAt: 123,
        answeredAt: undefined,
        title: "test topic",
        postfachId: "sjkdfhlksdfsdf",
        nachrichtEventId: "1",
        accessible: true,
        trustLevel: "LOW"
      },
      accessible: true,
      trustLevel: "LOW",
      status: "NEW",
      links: [{ rel: "topic", href: "http://test:123/topic" }]
    };
    const headerWithLastAnsweredDate: RueckfrageHeader = {
      ...rueckfrageHeader,
      nachrichtHeader: { ...rueckfrageHeader.nachrichtHeader, answeredAt: 456 }
    };

    test("returns rueckfrage header with id assigned", async () => {
      const mockFetchRueckfrageHeaders = jest
        .spyOn(axios, "get")
        .mockImplementation(() => Promise.resolve({ data: [rueckfrageHeader] }));
      const expectedRueckfrageHeader = {
        ...rueckfrageHeader,
        nachrichtHeader: {
          ...rueckfrageHeader.nachrichtHeader,
          sendAt: rueckfrageHeader.nachrichtHeader.sendAt,
          answeredAt: rueckfrageHeader.nachrichtHeader.answeredAt
        }
      };
      const rueckfrageHeaders = await getRueckfrageHeadersOfPostfach("postfachId");

      expect(rueckfrageHeaders.length).toEqual(1);
      expect(rueckfrageHeaders).toEqual([expectedRueckfrageHeader]);
      expect(mockFetchRueckfrageHeaders).toBeCalledTimes(1);
    });
    test("returns rueckfrage header with formatted lastAnsweredDate", async () => {
      const mockFetchRueckfrageHeaders = jest
        .spyOn(axios, "get")
        .mockImplementation(() => Promise.resolve({ data: [headerWithLastAnsweredDate] }));
      const expectedRueckfrageHeader = {
        ...headerWithLastAnsweredDate,
        nachrichtHeader: {
          ...headerWithLastAnsweredDate.nachrichtHeader,
          sendAt: headerWithLastAnsweredDate.nachrichtHeader.sendAt,
          answeredAt: headerWithLastAnsweredDate.nachrichtHeader.answeredAt!
        }
      };
      const rueckfrageHeaders = await getRueckfrageHeadersOfPostfach("postfachId");

      expect(rueckfrageHeaders.length).toEqual(1);
      expect(rueckfrageHeaders).toEqual([expectedRueckfrageHeader]);
      expect(mockFetchRueckfrageHeaders).toBeCalledTimes(1);
    });
    test("returns error message on error", async () => {
      const mockRueckfrageHeaders = jest.spyOn(axios, "get").mockReturnValue(Promise.reject());

      await expect(getRueckfrageHeadersOfPostfach("postfachId")).rejects.toEqual(Error(GET_RUECKFRAGE_HEADERS_FAILED));
      expect(mockRueckfrageHeaders).toBeCalledTimes(1);
    });
  });

  describe("get rueckfrage of postfach tests", () => {
    const rueckfrage: Rueckfrage = {
      id: "1",
      title: "test topic",
      nachricht: {
        id: "1",
        vorgangId: "1",
        sendAt: 123,
        title: "test topic",
        postfachId: "sjkdfhlksdfsdf",
        nachrichtEventId: "1",
        message: "test message",
        attachments: [],
        accessible: true,
        trustLevel: "LOW"
      },
      antworten: [],
      accessible: true,
      trustLevel: "LOW"
    };
    const rueckfrageWithAntwort: Rueckfrage = {
      ...rueckfrage,
      antworten: [
        {
          id: "2",
          rueckfrageId: "1",
          postfachId: "sjkdfhlksdfsdf",
          nachrichtEventId: "1",
          message: "test message",
          attachments: []
        }
      ]
    };

    test("returns rueckfrage with id assigned", async () => {
      const mockFetchRueckfrage = jest
        .spyOn(axios, "get")
        .mockImplementation(() => Promise.resolve({ data: rueckfrage }));
      const expectedRueckfrage = {
        ...rueckfrage,
        nachricht: {
          ...rueckfrage.nachricht,
          sendAt: rueckfrage.nachricht.sendAt
        }
      };
      const resultRueckfrage = await getRueckfrageOfPostfach("http://test.com");

      expect(resultRueckfrage).toEqual(expectedRueckfrage);
      expect(mockFetchRueckfrage).toBeCalledTimes(1);
    });
    test("returns rueckfrage with formatted user message date", async () => {
      const mockFetchRueckfrage = jest
        .spyOn(axios, "get")
        .mockImplementation(() => Promise.resolve({ data: rueckfrageWithAntwort }));
      const expectedRueckfrage = {
        ...rueckfrageWithAntwort,
        nachricht: {
          ...rueckfrageWithAntwort.nachricht,
          sendAt: rueckfrageWithAntwort.nachricht.sendAt,
          answeredAt: rueckfrageWithAntwort.nachricht.answeredAt
        }
      };
      const resultRueckfrage = await getRueckfrageOfPostfach("http://test.com");

      expect(resultRueckfrage).toEqual(expectedRueckfrage);
      expect(mockFetchRueckfrage).toBeCalledTimes(1);
    });
    test("returns error message on error", async () => {
      const mockFetchRueckfrage = jest.spyOn(axios, "get").mockReturnValue(Promise.reject());

      await expect(getRueckfrageOfPostfach("http://test.com")).rejects.toEqual(Error(GET_RUECKFRAGE_FAILED));
      expect(mockFetchRueckfrage).toBeCalledTimes(1);
    });
  });

  describe("sendMessage tests", () => {
    const commandReference: CommandReference = {
      id: "1",
      nachrichtEventId: "1"
    };
    const command: Command = {
      id: "1",
      nachrichtEventId: "1",
      status: "FINISHED",
      createdAt: 1
    };
    const newMessage: Antwort = {
      attachments: [],
      id: "1",
      rueckfrageId: "1",
      postfachId: "sjkdfhlksdfsdf",
      nachrichtEventId: "1",
      message: "New message"
    };
    test("returns the new message", async () => {
      const mockSendMessage = jest.spyOn(axios, "put").mockReturnValue(Promise.resolve({ data: commandReference }));
      const mockCheckSendStatus = jest.spyOn(axios, "get").mockReturnValue(Promise.resolve({ data: command }));
      const response = await sendMessage(newMessage);

      expect(response).toEqual(newMessage);
      expect(mockSendMessage).toBeCalledTimes(1);
      expect(mockCheckSendStatus).toBeCalledTimes(1);
    });
    test("throws error on send error", async () => {
      const mockSendMessage = jest.spyOn(axios, "put").mockReturnValue(Promise.reject());
      const mockCheckSendStatus = jest.spyOn(axios, "get").mockReturnValue(Promise.resolve({ data: command }));

      await expect(sendMessage(newMessage)).rejects.toEqual(Error(SEND_FAILED));
      expect(mockSendMessage).toBeCalledTimes(1);
      expect(mockCheckSendStatus).toBeCalledTimes(0);
    });
    test("throws error on status check error", async () => {
      const mockSendMessage = jest.spyOn(axios, "put").mockReturnValue(Promise.resolve({ data: commandReference }));
      const mockCheckSendStatus = jest.spyOn(axios, "get").mockReturnValue(Promise.reject());

      await expect(sendMessage(newMessage)).rejects.toEqual(Error(SEND_FAILED));
      expect(mockSendMessage).toBeCalledTimes(1);
      expect(mockCheckSendStatus).toBeCalledTimes(1);
    });
    test("throws error on error status", async () => {
      const mockSendMessage = jest.spyOn(axios, "put").mockReturnValue(Promise.resolve({ data: commandReference }));
      const mockCheckSendStatus = jest
        .spyOn(axios, "get")
        .mockReturnValue(Promise.resolve({ data: { ...command, status: "ERROR" } }));

      await expect(sendMessage(newMessage)).rejects.toEqual(Error(SEND_FAILED));
      expect(mockSendMessage).toBeCalledTimes(1);
      expect(mockCheckSendStatus).toBeCalledTimes(1);
    });
  });
});
