/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import * as React from "react";
import styled from "styled-components";
import { COLORS } from "../../constants";
import { transitionTimingStyles } from "../../commonStyles";
import { FaSpinner } from "react-icons/fa";

export interface ButtonProps extends React.ButtonHTMLAttributes<HTMLButtonElement> {
  /** Different, lighter visual style compared to a normal button */
  secondary?: boolean;
  /** If set, an icon will be desplayed next to the label */
  leftSideIcon?: React.ReactNode;
  /** If set, a spinner will be desplayed next to the label */
  showSpinner?: boolean;
}

/**
 * A simple button. Inherits all the props from html button element.
 */
export function Button({ secondary, leftSideIcon, showSpinner, ...props }: ButtonProps) {
  return (
    <StyledButton
      role="button"
      data-testid="button"
      aria-label={`${props.children}`}
      aria-disabled={props.disabled}
      $secondary={secondary}
      {...props}
    >
      {showSpinner && <FaSpinner className="spinner" />}
      {leftSideIcon && <IconWrapper>{leftSideIcon}</IconWrapper>}
      {props.children}
    </StyledButton>
  );
}

const IconWrapper = styled.div`
  margin-right: 0.5rem;
  height: 1rem;
  width: 1rem;
`;

const StyledButton = styled.button<{ $destructive?: boolean; $secondary?: boolean }>`
  align-items: center;
  background-color: ${(props) => (props.$secondary ? COLORS.LIGHT_GREY : COLORS.SLATE)};
  border-color: transparent;
  border-radius: 6px;
  border-style: none;
  border-width: 0;
  color: ${COLORS.WHITE};
  cursor: pointer;
  display: inline-flex;
  font-size: 0.875rem;
  font-weight: 500;
  height: 40px;
  width: fit-content;
  min-width: 120px;
  justify-content: center;
  line-height: 1.25rem;
  padding: 8px 16px;
  transition-property: color, background-color, border-color, text-decoration-color, fill, stroke;
  ${transitionTimingStyles};

  &:hover {
    /* e6 == 90% opacity in hex notation, cc == 80% opacity */
    background-color: ${(props) => (props.$secondary ? COLORS.LIGHT_GREY + "e6" : COLORS.SLATE + "cc")};
  }

  &:focus-visible {
    outline: none;
    box-shadow:
      0 0 0 2px ${COLORS.WHITE},
      0 0 0 4px ${(props) => (props.$secondary ? COLORS.LIGHT_GREY : COLORS.SLATE)};
  }

  &:disabled {
    pointer-events: none;
    opacity: 50%;
  }

  .spinner {
    animation: spin infinite 2s linear;
    margin-right: 8px;
  }

  @keyframes spin {
    from {
      transform: rotate(0deg);
    }
    to {
      transform: rotate(360deg);
    }
  }
`;
