/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import React, { KeyboardEvent } from "react";
import styled from "styled-components";
import { COLORS } from "../../constants";
import { transitionTimingStyles } from "../../commonStyles";

interface IconButtonProps extends React.HTMLAttributes<HTMLDivElement> {
  /** Action triggered after click. */
  onClick(): void;
  /** If the button should be displayed as disabled. */
  disabled?: boolean;
  /** The icon to be displayed. */
  icon: React.ReactNode;
  /** A descriptive label for the button for accessibility purposes. */
  label?: string;
}

/**
 * A button to be rendered as an icon button.<br>
 * At the moment following icons are included:
 * - **left-arow:** a left pointing black arrow
 * - **upload:** an upload icon
 *
 */
export function IconButton({ onClick, disabled, icon, label, ...props }: IconButtonProps) {
  const handleClick = () => {
    if (!disabled) {
      onClick();
    }
  };
  const handleKeyPress = (e: KeyboardEvent<HTMLDivElement>) => {
    if (!disabled && e.key === "Enter") {
      onClick();
    }
  };
  return (
    <Button
      aria-label={label}
      aria-disabled={disabled}
      tabIndex={0}
      role="button"
      data-testid="icon-button"
      disabled={disabled}
      onClick={handleClick}
      onKeyDown={handleKeyPress}
      {...props}
    >
      {icon}
    </Button>
  );
}

const Button = styled.div<{ disabled?: boolean }>`
  align-items: center;
  border: 1px solid ${COLORS.LAVENDER_MIST};
  border-radius: 6px;
  color: ${(props) => (props.disabled ? COLORS.LAVENDER_MIST : "black")};
  display: flex;
  font-weight: bold;
  height: 24px;
  justify-content: center;
  width: 24px;
  text-decoration: none;
  padding: 8px;
  cursor: ${(props) => (props.disabled ? "default" : "pointer")};
  transition-property: background-color;
  ${transitionTimingStyles};

  &:hover {
    background-color: ${(props) => (props.disabled ? "inherit" : COLORS.WHITE_SMOKE)};
  }
`;
