/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import React from "react";
import styled from "styled-components";
import { COLORS } from "../../constants";
import { AttachmentList, AttachmentItem } from "../../Data Display/AttachmentList/AttachmentList";
import { oneColumnGridContainer, twoColumnGridContainer } from "../../commonStyles";
import * as uuid from "uuid";

const cyIdsPrefix = "message-body-";

interface MessageBodyProps extends React.HTMLAttributes<HTMLDivElement> {
  nachrichtEventId: string;
  nachrichtId: string;
  message: string;
  attachments: AttachmentItem[];
  // optionally pass an index that will be included in this component's data-cy attributes
  index?: number;
  /** Accessibility label for message body */
  ariaLabel: string;
  /** Accessibility label for the close icon */
  closeIconLabel: string;
}

/**
 * A component to display the body of a message in a container with minimum height.
 */
export function MessageBody({
  nachrichtEventId,
  nachrichtId,
  message,
  attachments,
  index,
  ariaLabel,
  closeIconLabel,
  ...props
}: MessageBodyProps) {
  const tokens = message.split("\n");
  return (
    <StyledBody {...props} $hasAttachment={attachments && !!attachments.length}>
      <InnerText
        data-cy={`${cyIdsPrefix}${index !== undefined ? index + "-" : ""}content`}
        aria-labelledby={ariaLabel}
        data-testid="message-body-text"
      >
        {tokens.map((token, index) => {
          return (
            <span key={uuid.v4()}>
              {token}
              {index < tokens.length - 1 && <br />}
            </span>
          );
        })}
      </InnerText>
      <div>
        {attachments && !!attachments.length && (
          <AttachmentList
            nachrichtEventId={nachrichtEventId}
            nachrichtId={nachrichtId}
            attachments={attachments}
            closeIconLabel={closeIconLabel}
          />
        )}
      </div>
    </StyledBody>
  );
}

const StyledBody = styled.div<{ $hasAttachment: boolean }>`
  border: 1px solid ${COLORS.LAVENDER_MIST};
  border-radius: 12px;
  padding: 24px;
  min-height: 272px;
  ${(props) => (props.$hasAttachment ? twoColumnGridContainer : oneColumnGridContainer)}
`;

const InnerText = styled.div`
  font-size: 1rem;
  max-width: 720px;
`;
