/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import React, { SyntheticEvent } from "react";
import { fireEvent, render, screen } from "@testing-library/react";
import userEvent from "@testing-library/user-event";
import { AttachmentItem } from "../../Data Display/AttachmentList/AttachmentList";
import { ResponseForm } from "../../Compositions/ResponseForm/ResponseForm";

describe("Response form tests", () => {
  let attachmentFiles: File[];
  let fileCounter = 1;
  let attachments: AttachmentItem[];
  let message: string;
  let errorMessage: string;
  let onUpload: (e: React.ChangeEvent<HTMLInputElement>) => void;
  let onRemoveAttachment: (fileName: string) => void;
  let onWrite: (message: string) => void;
  let onSendResponse: (e: SyntheticEvent) => void;
  let response: { message: string; attachments: AttachmentItem[] } | undefined;

  beforeEach(() => {
    attachmentFiles = [
      new File(["something"], "file1.pdf", { type: "application/pdf" }),
      new File(["something2"], "file2.xlsx", {
        type: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
      })
    ];
    attachments = [];
    message = "";
    errorMessage = "";
    response = undefined;

    onUpload = (e) => {
      if (e.target.files && e.target.files.length > 0) {
        const newFile = e.target.files[0];
        const nextAttachment = {
          id: `${fileCounter++}`,
          fileName: newFile.name,
          fileSize: newFile.size,
          contentType: newFile.type
        };
        attachments = [...attachments, nextAttachment];
        errorMessage = "";
        e.target.value = "";
      }
    };

    onRemoveAttachment = (fileName: string) => {
      attachments = attachments.filter((a) => a.fileName !== fileName);
    };

    onWrite = (m: string) => {
      message = m;
    };

    onSendResponse = (e: SyntheticEvent) => {
      e.preventDefault();
      if (!message.length && !attachments.length) {
        errorMessage = "Fill all required fields";
        return;
      }
      response = {
        message,
        attachments: attachments
      };
    };
  });

  test("sends a text response", () => {
    render(
      <ResponseForm
        message={message}
        errorMessage={errorMessage}
        attachments={attachments}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        closeIconLabel="close"
        userInputAriaLabel="user input"
        sendingButtonLabel="Wird gesendet"
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );

    fireEvent.change(screen.getByTestId("textarea"), { target: { value: "text respose" } });
    fireEvent.click(screen.getByTestId("submit-button"));

    expect(response).toEqual({ message: "text respose", attachments: [] });
  });

  test("upload button click triggers input click", () => {
    const onClickSpy = jest.spyOn(HTMLInputElement.prototype, "click");
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        errorMessage={errorMessage}
        attachments={attachments}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );
    fireEvent.click(screen.getByTestId("upload-button"));
    expect(onClickSpy).toHaveBeenCalledTimes(1);
  });

  test("sends text response with attachments", () => {
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        errorMessage={errorMessage}
        attachments={attachments}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );

    fireEvent.change(screen.getByTestId("textarea"), { target: { value: "text respose" } });
    // while it would be nicer to emulate the button click, we have to trigger the upload on the input directly
    userEvent.upload(screen.getByTestId("attachment-input"), attachmentFiles[0]);
    userEvent.upload(screen.getByTestId("attachment-input"), attachmentFiles[1]);
    fireEvent.click(screen.getByTestId("submit-button"));

    expect(response).toEqual({
      message: "text respose",
      attachments: [
        {
          id: "1",
          fileName: "file1.pdf",
          fileSize: 9,
          contentType: "application/pdf"
        },
        {
          id: "2",
          fileName: "file2.xlsx",
          fileSize: 10,
          contentType: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        }
      ]
    });
  });

  test("attachments are displayed", () => {
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        errorMessage="Fill all required fields"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        attachments={[
          {
            id: "4",
            fileName: "file3.docx",
            fileSize: 1048576,
            contentType: "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
          },
          {
            id: "5",
            fileName: "file2.xlsx",
            fileSize: 2097152,
            contentType: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
          }
        ]}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );

    expect(screen.getAllByTestId("file-name").at(0)).toHaveTextContent("file3.docx");
    expect(screen.getAllByTestId("file-name").at(1)).toHaveTextContent("file2.xlsx");
  });

  test("displays error message after submit", () => {
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        errorMessage={errorMessage}
        attachments={attachments}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );

    fireEvent.click(screen.getByTestId("submit-button"));

    expect(errorMessage).toBe("Fill all required fields");
  });

  test("displays error message from props", () => {
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        errorMessage="Fill all required fields"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        attachments={attachments}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );

    expect(screen.getByTestId("error-content")).toHaveTextContent("Fill all required fields");
  });

  test("displays success message from props", () => {
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        successMessage="Erfolgreich gesendet"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        attachments={attachments}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
      />
    );

    expect(screen.getByTestId("success-content")).toHaveTextContent("Erfolgreich gesendet");
  });

  test("buttons disabled during sending", () => {
    render(
      <ResponseForm
        message={message}
        uploadButtonLabel="Datei hinzufügen"
        submitButtonLabel="Antwort absenden"
        uploadIconLabel="Hochladen"
        userInputAriaLabel="user input"
        closeIconLabel="close"
        sendingButtonLabel="Wird gesendet"
        attachments={attachments}
        onRemoveAttachment={onRemoveAttachment}
        onSendResponse={onSendResponse}
        onUpload={onUpload}
        onWrite={onWrite}
        isSending={true}
      />
    );

    expect(screen.getByTestId("submit-button")).toBeDisabled();
    expect(screen.getByTestId("upload-button")).toBeDisabled();
  });
});
