/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import React, { SyntheticEvent, useRef } from "react";
import { Frame } from "../../Data Display/Frame/Frame";
import { TextArea } from "../../Data Entry/TextArea/TextArea";
import { Button } from "../../Buttons/Button/Button";
import { AttachmentItem, AttachmentList } from "../../Data Display/AttachmentList/AttachmentList";
import styled from "styled-components";
import { Text } from "../../Data Display/Text/Text";
import { FaUpload } from "react-icons/fa";

const cyIdsPrefix = "response-form-";

interface ResponseFormProps {
  /** The message value. */
  message: string;
  /** A list with the uploaded attachments. */
  attachments: AttachmentItem[];
  /** The validation error message. */
  errorMessage?: string;
  /** The validation success message. */
  successMessage?: string;
  /** The label of the upload button. */
  uploadButtonLabel: string;
  /** The aria label for the upload icon. */
  uploadIconLabel: string;
  /** The label of the submit button. */
  submitButtonLabel: string;
  /** The label of the submit button while the response is sent. */
  sendingButtonLabel: string;
  /** Accessibility label for the close icon */
  closeIconLabel: string;
  /** The placeholder for the user message. */
  messagePlaceholder?: string;
  /** Accessibility label for the user response input */
  userInputAriaLabel: string;
  /** Function to control the upload of attachments. */
  onUpload(e: React.ChangeEvent<HTMLInputElement>): void;
  /** Function to control the removal of an attachment. */
  onRemoveAttachment(fileId: string): void;
  /** Function to send the response. */
  onSendResponse(e: SyntheticEvent): void;
  /** Function to control the message value on write. */
  onWrite(message: string): void;
  /** Set to true if the response is currently being sent to the backend. */
  isSending?: boolean;
}

/**
 * A response form for the user to enter a message and upload attachments.
 */
export function ResponseForm(props: ResponseFormProps) {
  const inputRef = useRef<HTMLInputElement>();

  let errorOrSuccessTextId;
  if (props.errorMessage) {
    errorOrSuccessTextId = "error_description";
  } else if (props.successMessage) {
    errorOrSuccessTextId = "success_description";
  }

  const isUploading = props.attachments.some((attachment) => attachment.isUploading);

  return (
    <Frame data-cy={`${cyIdsPrefix}wrapper`}>
      <FlexContainer>
        <TextArea
          data-cy={`${cyIdsPrefix}input`}
          value={props.message}
          onChangeValue={props.onWrite}
          aria-label={props.userInputAriaLabel}
          aria-required="true"
          placeholder={props.messagePlaceholder}
          aria-invalid={!!props.errorMessage?.length}
          aria-describedby={errorOrSuccessTextId}
        />
        {props.successMessage && (
          <SuccessText
            id={errorOrSuccessTextId}
            data-cy={`${cyIdsPrefix}success-msg`}
            dataTestId="success-content"
            variant="info"
          >
            {props.successMessage}
          </SuccessText>
        )}
        {props.errorMessage && (
          <Text
            role="alert"
            id={errorOrSuccessTextId}
            data-cy={`${cyIdsPrefix}error-msg`}
            dataTestId="error-content"
            variant="error"
          >
            {props.errorMessage}
          </Text>
        )}
      </FlexContainer>
      <FlexContainer>
        <StyledLabel>
          <Button
            data-cy={`${cyIdsPrefix}attachment-upload-btn`}
            type="button"
            data-testid="upload-button"
            secondary
            onClick={(event) => {
              event.preventDefault();
              if (inputRef.current) {
                inputRef.current.click();
              }
            }}
            leftSideIcon={<FaUpload title={props.uploadIconLabel} aria-label={props.uploadIconLabel} />}
            disabled={props.isSending}
          >
            {props.uploadButtonLabel}
          </Button>
          <input
            ref={(ref) => {
              if (ref) {
                inputRef.current = ref;
              }
            }}
            type="file"
            data-testid="attachment-input"
            style={{ display: "none" }}
            tabIndex={-1}
            onChange={props.onUpload}
          />
        </StyledLabel>

        {props.attachments && (
          <AttachmentList
            attachments={props.attachments}
            closeIconLabel={props.closeIconLabel}
            onRemove={props.onRemoveAttachment}
            disableDeleteButton={props.isSending}
          />
        )}
      </FlexContainer>
      <Button
        disabled={props.isSending || isUploading}
        data-cy={`${cyIdsPrefix}submit-btn`}
        type="submit"
        data-testid="submit-button"
        onClick={props.onSendResponse}
        showSpinner={props.isSending}
      >
        {props.isSending ? props.sendingButtonLabel : props.submitButtonLabel}
      </Button>
    </Frame>
  );
}

const FlexContainer = styled.form`
  display: flex;
  flex-direction: column;
  gap: 10px;
`;

const StyledLabel = styled.label`
  width: fit-content;
`;

const SuccessText = styled(Text)`
  color: green;
`;
