/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import React, { useState } from "react";
import styled from "styled-components";
import { Divider } from "../Divider/Divider";
import { useViewportDimensions } from "../../../hooks/useViewportDimensions";
import { tabletMinWidth } from "../../constants";
import { DropdownMenu, DropdownMenuHeading, DropdownMenuItem } from "../Dropdown Menu/DropdownMenu";
import { logout } from "../../../api/auth/auth.service";
import { NavigateFunction, useNavigate } from "react-router-dom";

const cyIdsPrefix = "application-header";

export interface HeaderProps {
  userName: string;
  pageHeaderAriaLabel: string;
  logoAltText: string;
  userIconAltText: string;
  bayernIdIconAltText: string;
}

/**
 * The header of the application.
 */
export function ApplicationHeader(props: HeaderProps) {
  const navigate: NavigateFunction = useNavigate();
  const viewportDimensions = useViewportDimensions();
  const antragsraumLogo = require("../../../assets/logo_antragsraum_weiss.png") as string;
  const userIcon = require("../../../assets/user_icon.png") as string;
  const bayernIdLogo = require("../../../assets/logo_bayernid.png") as string;
  const logoutIcon = require("../../../assets/log-out.svg") as string;
  const [dropdownMenuOpen, setDropdownMenuOpen] = useState(false);

  const userIconElement = <UserIcon data-cy={`${cyIdsPrefix}-user-icon`} src={userIcon} alt={props.userIconAltText} />;

  const fullUserInfo = (
    <UserInfo>
      <Row>
        <UserName data-testid="user-name" data-cy={`${cyIdsPrefix}-user-name`}>
          {props.userName}
        </UserName>
        {userIconElement}
      </Row>
      <Row>
        <BayernIdIcon
          data-testid="bayernid-icon"
          data-cy={`${cyIdsPrefix}-bayernid-icon`}
          src={bayernIdLogo}
          alt={props.bayernIdIconAltText}
        />
      </Row>
    </UserInfo>
  );

  const dropDownMenuItems = [
    ...(viewportDimensions.width < tabletMinWidth
      ? [<DropdownMenuHeading key="dropdown-heading">{fullUserInfo}</DropdownMenuHeading>]
      : []),
    <DropdownMenuItem
      key="dropdown-logout"
      handleClick={() => logout()}
      icon={<img src={logoutIcon} alt="" role="presentation" height={16} />}
      text="Logout"
    />
  ];

  return (
    <>
      <HeaderContainer data-cy={cyIdsPrefix}>
        <LeftSideContainer>
          <AccessibilityHeader aria-labelledby={props.pageHeaderAriaLabel}>
            <StyledImg
              onClick={() => {
                navigate("/");
              }}
              data-cy={`${cyIdsPrefix}-antragsraum-logo`}
              src={antragsraumLogo}
              alt={props.logoAltText}
            />
          </AccessibilityHeader>
        </LeftSideContainer>
        <RightSideContainer>
          {
            <UserInfoWrapper
              tabIndex={0}
              role="button"
              aria-haspopup="menu"
              aria-expanded={dropdownMenuOpen}
              aria-controls="dropdown-menu"
              onKeyDown={(e) => e.key === "Enter" && setDropdownMenuOpen(!dropdownMenuOpen)}
              onClick={() => setDropdownMenuOpen(!dropdownMenuOpen)}
            >
              {viewportDimensions.width < tabletMinWidth ? userIconElement : fullUserInfo}
            </UserInfoWrapper>
          }
          {dropdownMenuOpen ? <DropdownMenu listItems={dropDownMenuItems} /> : undefined}
        </RightSideContainer>
      </HeaderContainer>
      <Divider />
    </>
  );
}

/**
 * This header component is meant for the screen readers
 */
const AccessibilityHeader = styled.h1`
  display: inline-block;
  margin: 0;
`;

const HeaderContainer = styled.header`
  display: flex;
  justify-content: space-between;
`;

const LeftSideContainer = styled.div`
  flex-grow: 1;
  flex-shrink: 1;
`;

const RightSideContainer = styled.div`
  flex-shrink: 0;
  position: relative;
`;

const UserInfoWrapper = styled.div`
  cursor: pointer;
`;

const UserInfo = styled.div`
  display: flex;
  flex-direction: column;
  align-items: flex-end;
`;

const Row = styled.div`
  display: flex;
  align-items: center;
`;

const UserName = styled.span`
  font-size: 1.25rem;
  font-weight: 700;
  margin: 0 0.125rem 0 0;
`;

const UserIcon = styled.img`
  height: 32px;
`;

const BayernIdIcon = styled.img`
  height: 16px;
  margin-top: 0.25rem;
  padding: 0 3px;
`;

const StyledImg = styled.img`
  max-width: 100%;
`;
