/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import React from "react";
import styled from "styled-components";
import { COLORS } from "../../constants";
import * as uuid from "uuid";

export interface DropdownMenuProps extends React.HTMLAttributes<HTMLDivElement> {
  listItems: React.ReactNode[];
}

export function DropdownMenu({ listItems, ...props }: DropdownMenuProps) {
  const items = listItems.flatMap((e) => [e, <Divider key={`divider-${uuid.v4()}`} />]).slice(0, -1);

  return (
    <DropdownMenuWrapper {...props} role="menu">
      <DropdownMenuContainer>{items}</DropdownMenuContainer>
    </DropdownMenuWrapper>
  );
}

const DropdownMenuWrapper = styled.div`
  position: absolute;
  padding: 10px 0;
  right: 0;
  width: fit-content;
  z-index: 10;
`;

const DropdownMenuContainer = styled.div`
  display: flex;
  flex-direction: column;
  align-items: center;
  border-radius: 6px;
  border: 1px solid ${COLORS.DIVIDER_GREY};
  background: #fff;
  box-shadow: 0 4px 6px 0 rgba(0, 0, 0, 0.09);
`;

const Divider = styled.hr`
  margin: 0;
  border: none;
  border-top: 1px solid ${COLORS.DIVIDER_GREY};
  width: 100%;
`;

export interface DropdownMenuItemProps extends React.HTMLAttributes<HTMLDivElement> {
  icon: React.ReactNode;
  text: string;
  handleClick?(): void;
}

export function DropdownMenuItem({ icon, text, handleClick, ...props }: DropdownMenuItemProps) {
  return (
    <MenuItemSection>
      <MenuItem
        data-testid="menu-item"
        tabIndex={0}
        $clickable={!!handleClick}
        {...props}
        onKeyDown={(e) => {
          if (e.key === "Enter") {
            handleClick?.();
          }
        }}
        onClick={() => {
          handleClick?.();
        }}
      >
        <ItemIcon>{icon}</ItemIcon>
        <ItemText>{text}</ItemText>
      </MenuItem>
    </MenuItemSection>
  );
}

export interface DropdownMenuHeadingProps extends React.PropsWithChildren {}

export function DropdownMenuHeading(props: DropdownMenuHeadingProps) {
  return (
    <MenuItemSection>
      <MenuItem data-testid="menu-heading" $clickable={false}>
        {props.children}
      </MenuItem>
    </MenuItemSection>
  );
}

const MenuItemSection = styled.div`
  display: flex;
  padding: 5px;
  flex-direction: column;
  align-items: flex-start;
`;

const MenuItem = styled.div<{ $clickable: boolean }>`
  display: flex;
  min-width: 120px;
  padding: 6px 8px;
  align-items: center;
  gap: 8px;
  cursor: ${(props) => (props.$clickable ? "pointer" : "unset")};

  &:hover {
    background-color: ${(props) => (props.$clickable ? COLORS.WHITE_SMOKE : "unset")};
  }
`;

const ItemIcon = styled.div`
  width: 16px;
  height: 16px;
  flex-shrink: 0;
`;

const ItemText = styled.div`
  flex: 1 0 0;

  color: ${COLORS.SLATE_700};
  font-size: 0.875rem;
  font-weight: 500;
  line-height: 1.25rem;
`;
