/*
 * Copyright (c) 2023. Das Land Schleswig-Holstein vertreten durch den Ministerpräasidenten des
 * Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import React from "react";
import { fireEvent, render, screen } from "@testing-library/react";
import { AttachmentItem, AttachmentList } from "./AttachmentList";

describe("Attachment tests", () => {
  test("displays file name and size", () => {
    render(
      <AttachmentList
        nachrichtEventId="1"
        nachrichtId="2"
        attachments={[{ id: "1", fileName: "file.pdf", fileSize: 1572864, contentType: "application/pdf" }]}
      />
    );

    expect(screen.getByTestId("file-name")).toHaveTextContent("file.pdf");
    expect(screen.getByTestId("text-content")).toHaveTextContent("1.5 MB");
  });

  test("displays a PDF file with PDF icon", () => {
    render(
      <AttachmentList
        nachrichtEventId="1"
        nachrichtId="2"
        attachments={[{ id: "2", fileName: "file.pdf", fileSize: 1572864, contentType: "application/pdf" }]}
      />
    );

    expect(screen.getByTestId("pdf-icon")).toBeDefined();
  });

  test("displays a Word file with DOC icon", () => {
    render(
      <AttachmentList
        nachrichtEventId="1"
        nachrichtId="2"
        attachments={[
          {
            id: "3",
            fileName: "file.docx",
            fileSize: 1572864,
            contentType: "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
          }
        ]}
      />
    );

    expect(screen.getByTestId("doc-icon")).toBeDefined();
  });

  test("displays an Excel file with XLS icon", () => {
    render(
      <AttachmentList
        nachrichtEventId="1"
        nachrichtId="2"
        attachments={[
          {
            id: "4",
            fileName: "file.xlsx",
            fileSize: 1572864,
            contentType: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
          }
        ]}
      />
    );

    expect(screen.getByTestId("xls-icon")).toBeDefined();
  });

  test("displays all other files with a file icon", () => {
    render(
      <AttachmentList
        nachrichtEventId="1"
        nachrichtId="2"
        attachments={[{ id: "5", fileName: "file.png", fileSize: 1572864, contentType: "image/png" }]}
      />
    );

    expect(screen.getByTestId("file-icon")).toBeDefined();
  });

  test("multiple attachments", () => {
    render(
      <AttachmentList
        nachrichtEventId="1"
        nachrichtId="2"
        attachments={[
          { id: "6", fileName: "file1.png", fileSize: 1572864, contentType: "image/png" },
          { id: "7", fileName: "file2.png", fileSize: 1572864, contentType: "image/png" },
          { id: "8", fileName: "file3.png", fileSize: 1572864, contentType: "image/png" }
        ]}
      />
    );

    expect(screen.getAllByTestId("file-name").at(0)).toHaveTextContent("file1.png");
    expect(screen.getAllByTestId("file-name").at(1)).toHaveTextContent("file2.png");
    expect(screen.getAllByTestId("file-name").at(2)).toHaveTextContent("file3.png");
  });

  test("Remove attachments by click", () => {
    let attachments: AttachmentItem[] = [
      { id: "9", fileName: "file1.png", fileSize: 1572864, contentType: "image/png" },
      { id: "10", fileName: "file2.png", fileSize: 1572864, contentType: "image/png" }
    ];
    const onRemove = (fileId: string) => {
      attachments = attachments.filter((a) => a.id !== fileId);
    };
    render(<AttachmentList nachrichtEventId="1" nachrichtId="2" attachments={attachments} onRemove={onRemove} />);
    fireEvent.click(screen.getAllByTestId("close").at(0)!!);

    expect(attachments).toHaveLength(1);
    expect(attachments[0].fileName).toBe("file2.png");
  });

  test("Remove attachments by keaydown", () => {
    let attachments: AttachmentItem[] = [
      { id: "11", fileName: "file1.png", fileSize: 1572864, contentType: "image/png" },
      { id: "12", fileName: "file2.png", fileSize: 1572864, contentType: "image/png" }
    ];
    const onRemove = (fileId: string) => {
      attachments = attachments.filter((a) => a.id !== fileId);
    };
    render(<AttachmentList nachrichtEventId="1" nachrichtId="2" attachments={attachments} onRemove={onRemove} />);
    fireEvent.keyDown(screen.getAllByTestId("close").at(0)!!, { key: "Enter", code: 13, charCode: 13 });

    expect(attachments).toHaveLength(1);
    expect(attachments[0].fileName).toBe("file2.png");
  });
});
