/*
 * Copyright (c) 2023-2024.  Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import React, { useState } from "react";
import styled from "styled-components";
import { COLORS } from "../../constants";
import { FaFile, FaFileExcel, FaFilePdf, FaFileWord, FaSpinner } from "react-icons/fa";
import { AiOutlineClose, AiFillFileExclamation } from "react-icons/ai";
import { Text } from "../Text/Text";
import { downloadAttachment, saveDownloadedAttachmentToFile } from "../../../api/attachments/attachmentsAPI";

const cyListPrefix = "attachment-list-";
const cyAttachmentPrefix = `${cyListPrefix}item-`;

export interface AttachmentItem {
  id: string;
  fileName: string;
  fileSize: number; // in bytes
  contentType: string;
  isUploading?: boolean;
  errorMessage?: string;
}

interface AttachmentListProps {
  /** An attachment item consists mainly of a file name and size. */
  attachments: AttachmentItem[];
  /** The id of the nachricht event the nachricht header belongs to */
  nachrichtEventId?: string;
  /** The id of the nachricht the attachment belongs to */
  nachrichtId?: string;
  /** If this function is defined the attachments can be removed. */
  onRemove?: (fileName: string) => void;
  /** Accessibility label for the close icon */
  closeIconLabel?: string;
  disableDeleteButton?: boolean;
}

interface AttachmentProps extends React.HTMLAttributes<HTMLDivElement> {
  nachrichtEventId?: string;
  nachrichtId?: string;
  attachment: AttachmentItem;
  onRemove?: (fileId: string) => void;
  hasBorder?: boolean;
  closeIconLabel?: string;
  disableDeleteButton?: boolean;
}

/**
 * Renders attachments with the respective icon based on the file type.
 */
export function AttachmentList(props: AttachmentListProps) {
  return (
    <StyledFrame
      data-cy={`${cyListPrefix}wrapper`}
      style={{ border: props.attachments.length === 0 ? "none" : `1px solid ${COLORS.LAVENDER_MIST}` }}
    >
      {props.attachments.map((attachment, i) => (
        <Attachment
          data-cy={`${cyAttachmentPrefix}${i}`}
          attachment={attachment}
          onRemove={props.onRemove}
          hasBorder={i !== 0}
          key={attachment.fileName}
          closeIconLabel={props.closeIconLabel}
          nachrichtEventId={props.nachrichtEventId}
          nachrichtId={props.nachrichtId}
          disableDeleteButton={props.disableDeleteButton || attachment.isUploading}
        />
      ))}
    </StyledFrame>
  );
}

function formatFileSize(sizeInBytes: number): string {
  const units = ["Bytes", "KB", "MB"];
  const unitToUseIndex = Math.floor(Math.log(sizeInBytes) / Math.log(1024));
  return `${parseFloat((sizeInBytes / Math.pow(1024, unitToUseIndex)).toFixed(1))} ${units[unitToUseIndex]}`;
}

function Attachment({
  attachment,
  onRemove,
  hasBorder,
  closeIconLabel,
  nachrichtEventId,
  nachrichtId,
  disableDeleteButton,
  ...props
}: AttachmentProps) {
  const [isDownloading, setIsDownloading] = useState<boolean>(false);

  const attachmentContainer = (
    <AttachmentContainer {...props} style={{ borderTop: hasBorder ? `1px solid ${COLORS.LAVENDER_MIST}` : "none" }}>
      <FileNameWithIcon>
        <FileIcon
          fileName={attachment.fileName}
          showSpinner={attachment.isUploading || isDownloading}
          error={!!attachment.errorMessage}
        />
        <FileNameWithSize>
          <FileName data-cy={`${cyAttachmentPrefix}file-name`} data-testid="file-name">
            {attachment.fileName}
          </FileName>
          {!attachment.errorMessage && (
            <Text data-cy={`${cyAttachmentPrefix}file-size`} variant="info">
              {formatFileSize(attachment.fileSize)}
            </Text>
          )}
          {attachment.errorMessage && (
            <Text data-cy={`${cyAttachmentPrefix}error-message`} variant="error">
              {attachment.errorMessage}
            </Text>
          )}
        </FileNameWithSize>
      </FileNameWithIcon>
      {!!onRemove && !disableDeleteButton && (
        <CloseIcon
          data-cy={`${cyAttachmentPrefix}close-btn`}
          data-testid="close"
          role="button"
          aria-label={closeIconLabel}
          tabIndex={0}
          onKeyDown={(e) => {
            if (e.key === "Enter") {
              onRemove(attachment.id);
            }
          }}
          onClick={() => onRemove(attachment.id)}
        >
          <AiOutlineClose />
        </CloseIcon>
      )}
    </AttachmentContainer>
  );

  return (
    <>
      {nachrichtEventId && nachrichtId ? (
        <StyledLink
          $disabled={isDownloading}
          onClick={() => {
            if (!isDownloading) {
              setIsDownloading(true);
              downloadAttachment(nachrichtEventId, nachrichtId, attachment.id)
                .then((blob) => saveDownloadedAttachmentToFile(blob, attachment.fileName))
                .catch(() => {}) // TODO: no error handling is specified
                .finally(() => setIsDownloading(false));
            }
          }}
        >
          {attachmentContainer}
        </StyledLink>
      ) : (
        attachmentContainer
      )}
    </>
  );
}

const AttachmentContainer = styled.div`
  align-items: center;
  display: flex;
  justify-content: space-between;
  padding: 8px 12px;
`;

interface FileIconProps {
  fileName: string;
  showSpinner?: boolean;
  error?: boolean;
}

function FileIcon(props: FileIconProps) {
  const isPdfFile = !props.error && !props.showSpinner && props.fileName.toLowerCase().endsWith(".pdf");
  const isWordFile =
    !props.error &&
    !props.showSpinner &&
    (props.fileName.toLowerCase().endsWith(".doc") || props.fileName.toLowerCase().endsWith(".docx"));
  const isExcelFile =
    !props.error &&
    !props.showSpinner &&
    (props.fileName.toLowerCase().endsWith(".xls") || props.fileName.toLowerCase().endsWith(".xlsx"));
  const isOtherFile = !props.error && !props.showSpinner && !isPdfFile && !isWordFile && !isExcelFile;

  return (
    <FileIconWrapper data-cy={`${cyAttachmentPrefix}file-icon`}>
      {props.error && (
        <AiFillFileExclamation
          title="Uploadfehler Symbol"
          aria-label="Uploadfehler Symbol"
          data-testid="upload-error-icon"
          color={COLORS.CARMINE_PINK}
        />
      )}
      {isPdfFile && <FaFilePdf title="PDF Symbol" aria-label="PDF Symbol" data-testid="pdf-icon" color={COLORS.PDF} />}
      {isWordFile && (
        <FaFileWord title="Word Symbol" aria-label="Word Symbol" data-testid="doc-icon" color={COLORS.DOCX} />
      )}
      {isExcelFile && (
        <FaFileExcel title="Excel Symbol" aria-label="Excel Symbol" data-testid="xls-icon" color={COLORS.XLSX} />
      )}
      {isOtherFile && (
        <FaFile title="Datei Symbol" aria-label="Datei Symbol" data-testid="file-icon" color={COLORS.SLATE} />
      )}
      {props.showSpinner && <FaSpinner className="spinner" />}
    </FileIconWrapper>
  );
}

const FileIconWrapper = styled.div`
  font-size: 2.5rem;

  .spinner {
    animation: spin infinite 2s linear;
  }

  @keyframes spin {
    from {
      transform: rotate(0deg);
    }
    to {
      transform: rotate(360deg);
    }
  }
`;

const StyledFrame = styled.div`
  display: flex;
  flex-direction: column;
  border-radius: 6px;
  border: 1px solid ${COLORS.DIVIDER_GREY};
  background-color: ${COLORS.ALABASTER};
  max-width: 523px;
`;

const FileNameWithIcon = styled.div`
  display: flex;
  gap: 10px;
`;

const FileNameWithSize = styled.div`
  display: flex;
  flex-direction: column;
`;

const FileName = styled.div`
  font-size: 1rem;
  line-break: anywhere;
`;

const CloseIcon = styled.div`
  font-size: 1.875rem;
  color: ${COLORS.SLATE};
  cursor: pointer;
`;

const StyledLink = styled.div<{ $disabled: boolean }>`
  color: unset;
  text-decoration: none;
  cursor: ${(props) => (props.$disabled ? "default" : "pointer")};
`;
