/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import * as api from "../../api/rueckfragen/rueckfragenAPI";
import { Antwort, Rueckfrage, RueckfrageHeader } from "../../api/types";
import { store } from "../../store";
import {
  clearSelectedRueckfrage,
  fetchRueckfrageHeadersThunk,
  fetchRueckfragenThunk,
  rueckfragenReducer,
  RueckfrageState,
  sendMessageThunk
} from "./rueckfrageSlice";

describe("rueckfragen reducer tests", () => {
  const initialState: RueckfrageState = {
    rueckfrageHeaderStatus: "idle",
    selectedRueckfrageStatus: "idle",
    rueckfrageHeaders: []
  };
  const postfachId = "28721c6f-b78f-4d5c-a048-19fd2fc429d2";
  const rueckfrageLink = "http://test:123/rueckfrage";
  const rueckfrageHeader: RueckfrageHeader = {
    id: "1",
    title: "test topic",
    nachrichtHeader: {
      id: "1",
      vorgangId: "1",
      sendAt: 123,
      title: "test topic",
      postfachId: "sjkdfhlksdfsdf",
      nachrichtEventId: "1",
      accessible: true,
      trustLevel: "LOW"
    },
    accessible: true,
    trustLevel: "LOW",
    status: "NEW",
    links: [{ rel: "rueckfrage", href: rueckfrageLink }]
  };
  const rueckfrage: Rueckfrage = {
    id: "1",
    title: "test topic",
    nachricht: {
      id: "1",
      vorgangId: "1",
      sendAt: 123,
      title: "test topic",
      postfachId: "sjkdfhlksdfsdf",
      nachrichtEventId: "1",
      message: "test message",
      attachments: [],
      accessible: true,
      trustLevel: "LOW"
    },
    antworten: [],
    accessible: true,
    trustLevel: "LOW"
  };
  const loadedState: RueckfrageState = {
    rueckfrageHeaderStatus: "succeeded",
    selectedRueckfrageStatus: "succeeded",
    rueckfrageHeaders: [rueckfrageHeader],
    selectedRueckfrage: rueckfrage
  };

  it("should handle initial state", () => {
    expect(rueckfragenReducer(undefined, { type: "unknown" })).toEqual(initialState);
  });
  it("should clear selected topic", () => {
    const actual = rueckfragenReducer(loadedState, clearSelectedRueckfrage());
    expect(actual).toEqual({
      ...loadedState,
      selectedRueckfrageStatus: "idle",
      selectedRueckfrage: undefined
    });
  });

  describe("fetchRueckfrageHeadersThunk tests", () => {
    it("calls getRueckfrageHeadersOfPostfach from api", () => {
      const mockFetchAllRueckfrageHeaders = jest
        .spyOn(api, "getRueckfrageHeadersOfPostfach")
        .mockReturnValue(Promise.resolve([]));
      store.dispatch(fetchRueckfrageHeadersThunk(postfachId));
      expect(mockFetchAllRueckfrageHeaders).toBeCalledTimes(1);
    });
    it("should fetch all rueckfrage headers successfully", () => {
      const action = fetchRueckfrageHeadersThunk.fulfilled([rueckfrageHeader], "", postfachId);
      const actual = rueckfragenReducer(initialState, action);
      expect(actual).toEqual({
        ...initialState,
        rueckfrageHeaderStatus: "succeeded",
        rueckfrageHeaders: [rueckfrageHeader]
      });
    });
    it("should have error state after failed topic header load", () => {
      const action = fetchRueckfrageHeadersThunk.rejected(null, "", postfachId); // error object is null
      const actual = rueckfragenReducer(initialState, action);
      expect(actual).toEqual({
        ...initialState,
        rueckfrageHeaderStatus: "failed"
      });
    });
    it("should keep previously available rueckfrage headers after a failed load", () => {
      const action = fetchRueckfrageHeadersThunk.rejected(null, "", postfachId); // error object is null
      const actual = rueckfragenReducer(loadedState, action);
      expect(actual).toEqual({
        ...loadedState,
        rueckfrageHeaderStatus: "failed"
      });
    });
  });

  describe("fetchRueckfrageThunk tests", () => {
    it("calls getRueckfragenOfPostfach from api", () => {
      const mockFetchRueckfrage = jest
        .spyOn(api, "getRueckfrageOfPostfach")
        .mockReturnValue(Promise.resolve(rueckfrage));
      store.dispatch(fetchRueckfragenThunk(rueckfrageLink));
      expect(mockFetchRueckfrage).toBeCalledTimes(1);
    });
    it("should fetch rueckfrage successfully", () => {
      const action = fetchRueckfragenThunk.fulfilled(rueckfrage, "", rueckfrageLink);
      const actual = rueckfragenReducer(initialState, action);
      expect(actual).toEqual({
        ...initialState,
        selectedRueckfrageStatus: "succeeded",
        selectedRueckfrage: rueckfrage
      });
    });
    it("should have error state after failed rueckfrage load", () => {
      const action = fetchRueckfragenThunk.rejected(null, "", rueckfrageLink); // error object is null
      const actual = rueckfragenReducer(initialState, action);
      expect(actual).toEqual({
        ...initialState,
        selectedRueckfrageStatus: "failed"
      });
    });
    it("should keep previously available rueckfrage after a failed load", () => {
      const action = fetchRueckfragenThunk.rejected(null, "", rueckfrageLink); // error object is null
      const actual = rueckfragenReducer(loadedState, action);
      expect(actual).toEqual({
        ...loadedState,
        selectedRueckfrageStatus: "failed"
      });
    });
  });

  describe("sendMessageThunk tests", () => {
    const newMessage: Antwort = {
      id: "2",
      rueckfrageId: "2",
      postfachId: "sjkdfhlksdfsdf",
      nachrichtEventId: "1",
      message: "Test",
      attachments: []
    };
    const answeredRueckfrage: Rueckfrage = {
      ...rueckfrage,
      antworten: [newMessage]
    };
    const initialStateForSendMessage: RueckfrageState = {
      rueckfrageHeaderStatus: "succeeded",
      selectedRueckfrageStatus: "succeeded",
      rueckfrageHeaders: [rueckfrageHeader, { ...rueckfrageHeader, id: "2" }],
      selectedRueckfrage: rueckfrage
    };

    it("should call sendMessage from api", () => {
      const mockMessage = {
        id: "3",
        rueckfrageId: "3",
        attachments: [],
        date: 12345,
        message: "",
        postfachId: "sjkdfhlksdfsdf",
        nachrichtEventId: "1"
      };
      const mockSendMessage = jest.spyOn(api, "sendMessage").mockReturnValue(Promise.resolve(mockMessage));
      store.dispatch(sendMessageThunk({ id: "topicId", newMessage: mockMessage }));
      expect(mockSendMessage).toBeCalledTimes(1);
      expect(mockSendMessage).toBeCalledWith(mockMessage);
    });
    it("should reset load status after successful message send", () => {
      const action = sendMessageThunk.fulfilled({ message: newMessage, id: "1" }, "", {
        id: "1",
        newMessage
      });
      const actual = rueckfragenReducer(initialStateForSendMessage, action);
      expect(actual).toEqual({
        ...initialStateForSendMessage,
        selectedRueckfrageStatus: "idle"
      });
    });
    it("failed message send doesn't affect state", () => {
      const action = sendMessageThunk.rejected(null, "", { id: "1", newMessage }); // error object is null
      const actual = rueckfragenReducer(initialStateForSendMessage, action);
      expect(actual).toEqual(initialStateForSendMessage);
    });
  });
});
