/*
 * Copyright (c) 2023-2024.  Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import React, { SyntheticEvent, useEffect } from "react";
import { MessageTitle } from "../../components/Compositions/MessageTitle/MessageTitle";
import { Text } from "../../components/Data Display/Text/Text";
import { MessageBody } from "../../components/Compositions/MessageBody/MessageBody";
import { Heading } from "../../components/Data Display/Heading/Heading";
import { AttachmentItem } from "../../components/Data Display/AttachmentList/AttachmentList";
import { ResponseForm } from "../../components/Compositions/ResponseForm/ResponseForm";
import { useNavigate, useParams } from "react-router-dom";
import { useTranslateFunction } from "../../hooks/useTranslate";
import { useAppDispatch, useAppSelector } from "../../hooks";
import {
  fetchRueckfragenThunk,
  resetRueckfrageHeaderStatus,
  sendMessageThunk
} from "../../features/rueckfragen/rueckfrageSlice";
import { Antwort } from "../../api/types";
import { LoadingPage } from "../../components/Core/Loading Page/LoadingPage";
import { ErrorPage } from "../../components/Core/Error Page/ErrorPage";
import { formatTimestampToDate, formatTimestampToDateAndTime } from "../../lib/date";
import { uploadAttachment } from "../../api/attachments/attachmentsAPI";
import styled from "styled-components";
import { unwrapResult } from "@reduxjs/toolkit";
import * as uuid from "uuid";
import { isRueckfrageMarkedNew } from "../../lib/rueckfragen/isRueckfrageMarkedNew";

const cyIdsPrefix = "detail-";

export function DetailPage() {
  const { encodedRueckfrageLink } = useParams();
  const { selectedRueckfrageStatus, selectedRueckfrage } = useAppSelector((state) => state.rueckfragen);
  const dispatch = useAppDispatch();

  const navigate = useNavigate();
  const translate = useTranslateFunction();

  const [message, setMessage] = React.useState("");
  const [successMessage, setSuccessMessage] = React.useState("");
  const [errorMessage, setErrorMessage] = React.useState("");
  const [attachments, setAttachments] = React.useState<AttachmentItem[]>([]);
  const [isSending, setIsSending] = React.useState(false);

  useEffect(() => {
    if (selectedRueckfrageStatus === "idle") {
      const decodedRueckfrageLink = atob(encodedRueckfrageLink!);

      dispatch(fetchRueckfragenThunk(decodedRueckfrageLink));
    }
  }, [selectedRueckfrageStatus, dispatch]);

  const onRemove = (fileId: string) => {
    const newAttachments = attachments.filter((a) => a.id !== fileId);
    setAttachments(newAttachments);
  };

  const onUpload = (e: React.ChangeEvent<HTMLInputElement>) => {
    if (e.target.files && e.target.files.length > 0 && selectedRueckfrage) {
      const fileToUpload = e.target.files[0];
      const fileUploadId = fileToUpload.name + "-" + new Date().getTime();

      setErrorMessage("");
      setSuccessMessage("");
      setAttachments((prevAttachments) => [
        ...prevAttachments,
        {
          id: fileUploadId,
          fileName: fileToUpload.name,
          fileSize: fileToUpload.size,
          contentType: fileToUpload.type,
          isUploading: true
        }
      ]);

      uploadAttachment(
        selectedRueckfrage.nachricht.vorgangId,
        selectedRueckfrage.nachricht.nachrichtEventId,
        fileToUpload
      )
        .then((fileId) => {
          setAttachments((prevAttachments) =>
            prevAttachments.map((attachment) => {
              if (attachment.id === fileUploadId) {
                return { ...attachment, id: fileId, isUploading: false };
              }

              return attachment;
            })
          );
        })
        .catch((error: Error) => {
          setAttachments((prevAttachments) =>
            prevAttachments.map((attachment) => {
              if (attachment.id === fileUploadId) {
                return { ...attachment, isUploading: false, errorMessage: translate(error.message) };
              }

              return attachment;
            })
          );
        });

      e.target.value = "";
    }
  };

  const onWrite = (m: string) => {
    setMessage(m);
    if (!!m.length) {
      setErrorMessage("");
      setSuccessMessage("");
    }
  };

  const onSendResponse = (e: SyntheticEvent) => {
    e.preventDefault();

    const uploadedAttachments = attachments.filter((attachment) => !attachment.errorMessage);
    if (!message.replace(/\s/g, "").length && !uploadedAttachments.length) {
      setSuccessMessage("");
      setErrorMessage(translate("detailPage.responseForm.emptyMessageValidation"));

      return; // the message should not be empty or contain only whitespaces
    }

    const newMessage: Antwort = {
      id: selectedRueckfrage!.nachricht.id,
      rueckfrageId: selectedRueckfrage!.nachricht.id,
      postfachId: selectedRueckfrage!.nachricht.postfachId,
      nachrichtEventId: selectedRueckfrage!.nachricht.nachrichtEventId,
      message,
      attachments: uploadedAttachments
    };

    if (selectedRueckfrage) {
      setIsSending(true);
      dispatch(sendMessageThunk({ newMessage, id: selectedRueckfrage.id! }))
        .then(unwrapResult)
        .then(() => {
          dispatch(resetRueckfrageHeaderStatus());
          setAttachments([]);
          setMessage("");
          setSuccessMessage(translate("detailPage.responseForm.sendSuccess"));
        })
        .catch(() => {
          setErrorMessage(translate("detailPage.responseForm.sendFailed"));
        })
        .finally(() => setIsSending(false));
    }
  };

  if (selectedRueckfrageStatus === "loading") {
    return <LoadingPage loadingMessage={translate("common.loadingMessage")} />;
  }

  if (selectedRueckfrageStatus === "failed") {
    return (
      <ErrorPage
        errorType="generic"
        message={translate("common.error.generic.message")}
        goBackTo={translate("common.error.goBackTo")}
        linkText={translate("common.error.linkText")}
        errorIconAltText={translate("common.error.errorIconAltText")}
        notFoundIconAltText={translate("common.error.notFoundIconAltText")}
      />
    );
  }

  if (!selectedRueckfrage) {
    return (
      <ErrorPage
        errorType="notFound"
        headline={translate("common.error.notFound.heading")}
        message={translate("common.error.notFound.message")}
        goBackTo={translate("common.error.goBackTo")}
        linkText={translate("common.error.linkText")}
        errorIconAltText={translate("common.error.errorIconAltText")}
        notFoundIconAltText={translate("common.error.notFoundIconAltText")}
      />
    );
  }

  document.title =
    translate("detailPage.htmlTitle", { formattedDate: formatTimestampToDate(selectedRueckfrage.nachricht.sendAt) }) +
    " | " +
    translate("common.htmlTitle");

  return (
    <>
      <GridDiv $gap={19}>
        <MessageTitle
          title={translate("detailPage.title", {
            formattedDate: formatTimestampToDateAndTime(selectedRueckfrage.nachricht.sendAt)
          })}
          subtitle={selectedRueckfrage.nachricht.title}
          unread={isRueckfrageMarkedNew(selectedRueckfrage)}
          badgeLabel={translate("common.new")}
          leftArrowIconLabel={translate("common.leftArrow")}
          onReturnClick={() => {
            navigate("/");
          }}
          disableReturnButton={isSending}
        />
        <Text data-cy={`${cyIdsPrefix}hint-text`} variant="info">
          {translate("detailPage.hint")}
        </Text>
      </GridDiv>
      <MessageBody
        nachrichtEventId={selectedRueckfrage.nachricht.nachrichtEventId}
        nachrichtId={selectedRueckfrage.nachricht.id}
        closeIconLabel={translate("common.close")}
        data-cy={`${cyIdsPrefix}clerk-message`}
        ariaLabel={translate("detailPage.messageBody")}
        message={selectedRueckfrage.nachricht.message}
        attachments={selectedRueckfrage.nachricht.attachments}
      />

      {selectedRueckfrage.antworten.map((response, i) => (
        <GridDiv $gap={12} data-cy={`${cyIdsPrefix}user-message-${i}-wrapper`} key={uuid.v4()}>
          <Heading headingLevel={2} data-cy={`${cyIdsPrefix}user-message-${i}-heading`} key={uuid.v4()}>
            {translate("detailPage.response.title")}
          </Heading>
          <MessageBody
            nachrichtEventId={response.nachrichtEventId}
            nachrichtId={response.id}
            closeIconLabel={translate("common.close")}
            data-cy={`${cyIdsPrefix}user-message-${i}`}
            ariaLabel={translate("detailPage.messageBody")}
            key={uuid.v4()}
            message={response.message}
            attachments={response.attachments}
            index={i}
          />
        </GridDiv>
      ))}
      <GridDiv $gap={12}>
        <Heading headingLevel={2} data-cy={`${cyIdsPrefix}response-heading`}>
          {translate("detailPage.responseForm.title")}
        </Heading>
        <ResponseForm
          message={message}
          successMessage={successMessage}
          errorMessage={errorMessage}
          attachments={attachments}
          uploadButtonLabel={translate("detailPage.uploadButton")}
          submitButtonLabel={translate("detailPage.submitButton")}
          messagePlaceholder={translate("detailPage.messagePlaceholder")}
          uploadIconLabel={translate("common.upload")}
          userInputAriaLabel={translate("detailPage.userInputAriaLabel")}
          closeIconLabel={translate("common.close")}
          sendingButtonLabel={translate("detailPage.responseForm.sending")}
          onRemoveAttachment={onRemove}
          onUpload={onUpload}
          onWrite={onWrite}
          onSendResponse={onSendResponse}
          isSending={isSending}
        />
      </GridDiv>
    </>
  );
}

const GridDiv = styled.div<{ $gap: number }>`
  display: grid;
  gap: ${(props) => props.$gap}px;
`;
