/*
 * Copyright (c) 2023-2024.  Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten
 * des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung.
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import React, { useEffect } from "react";
import { useAppDispatch, useAppSelector } from "../../hooks";
import { clearSelectedRueckfrage, fetchRueckfrageHeadersThunk } from "../../features/rueckfragen/rueckfrageSlice";
import { useTranslateFunction } from "../../hooks/useTranslate";
import { LoadingPage } from "../../components/Core/Loading Page/LoadingPage";
import { ErrorPage } from "../../components/Core/Error Page/ErrorPage";
import { getCurrentUserFromLocalStorage } from "../../api/auth/auth.service";
import OverviewList from "../../components/Compositions/OverviewList/OverviewList";
import LocaleButton from "../../components/Compositions/LocaleButton/LocaleButton";
import * as S from "./OverviewPageStyles";
import { getRuntimeEnvProps } from "../../lib/env/env";

const nothingToDoLogo: string = require("../../assets/check.svg") as string;
const showLocaleButton: boolean = getRuntimeEnvProps().LOCALE_BUTTON === "true";

export function OverviewPage() {
  const dispatch = useAppDispatch();
  const translate = useTranslateFunction();
  const user = getCurrentUserFromLocalStorage();
  const { rueckfrageHeaderStatus, rueckfrageHeaders } = useAppSelector((state) => state.rueckfragen);

  useEffect(() => {
    if (rueckfrageHeaderStatus === "idle") {
      if (user?.postkorbhandle) {
        dispatch(fetchRueckfrageHeadersThunk(user.postkorbhandle));
      } else {
        console.error("user token has no postkorb handle");
      }
    }
  }, [rueckfrageHeaderStatus, dispatch]);

  useEffect(() => {
    document.title = translate("overviewPage.htmlTitle") + " | " + translate("common.htmlTitle");
    dispatch(clearSelectedRueckfrage());
  }, [dispatch]);

  if (rueckfrageHeaderStatus === "loading") {
    return <LoadingPage loadingMessage={translate("common.loadingMessage")} />;
  }

  if (rueckfrageHeaderStatus === "failed") {
    return (
      <ErrorPage
        errorType="generic"
        message={translate("common.error.generic.message")}
        goBackTo={translate("common.error.goBackTo")}
        linkText={translate("common.error.linkText")}
        errorIconAltText={translate("common.error.errorIconAltText")}
        notFoundIconAltText={translate("common.error.notFoundIconAltText")}
      />
    );
  }

  return (
    <>
      {showLocaleButton && <LocaleButton />}
      <S.StyledDiv>
        <S.StyledHeading headingLevel={1} data-cy={`overview-heading`}>
          {translate("overviewPage.heading", { userName: `${user?.firstname ?? ""} ${user?.lastname ?? ""}` })}
        </S.StyledHeading>
        <S.StyledText data-cy={`overview-info-text`} variant="info">
          {rueckfrageHeaders.length === 0
            ? translate("overviewPage.infoText.empty")
            : translate("overviewPage.infoText.notEmpty")}
        </S.StyledText>
      </S.StyledDiv>
      {rueckfrageHeaders.length > 0 ? (
        <OverviewList />
      ) : (
        <S.NoMessagesContainer>
          <img
            width={64}
            height={64}
            data-cy={`overview-nothing-to-do-logo`}
            src={nothingToDoLogo}
            alt={translate("overviewPage.nothingToDoIconAltText")}
          />
          <S.NoMessagesText variant="info">{translate("overviewPage.nothingToDo")}</S.NoMessagesText>
        </S.NoMessagesContainer>
      )}
    </>
  );
}
